from threaded_messages.models import inbox_count_for

def inbox(request):
    if request.user.is_authenticated():
        return {'messages_inbox_count': inbox_count_for(request.user)}
    else:
        return {}



from django.conf.urls.defaults import *
from django.views.generic.simple import redirect_to

from threaded_messages.views import *

urlpatterns = patterns('',
    url(r'^$', redirect_to, {'url': 'inbox/'}),
    url(r'^search/$', search, name='messages_search'),
    url(r'^inbox/$', inbox, name='messages_inbox'),
    url(r'^outbox/$', outbox, name='messages_outbox'),
    url(r'^compose/$', compose, name='messages_compose'),
    url(r'^compose/(?P<recipient>[\+\w]+)/$', compose, name='messages_compose_to'),
    url(r'^view/(?P<thread_id>[\d]+)/$', view, name='messages_detail'),
    url(r'^delete/(?P<thread_id>[\d]+)/$', delete, name='messages_delete'),
    url(r'^undelete/(?P<thread_id>[\d]+)/$', undelete, name='messages_undelete'),
    url(r'^batch-update/$', batch_update, name='messages_batch_update'),
    url(r'^trash/$', trash, name='messages_trash'),
    
    url(r"^recipient-search/$", recipient_search, name="recipient_search"),
    url(r'^message-reply/(?P<thread_id>[\d]+)/$', message_ajax_reply, name="message_reply"),
    
    # modal composing 
    url(r'^modal-compose/(?P<recipient>[\w.+-_]+)/$', compose, {
                            "template_name":"django_messages/modal_compose.html",
                            "form_class": ComposeForm
                        }, name='modal_messages_compose_to'),
    
    url(r'^modal-compose/$', compose, {
                            "template_name":"django_messages/modal_compose.html",
                            "form_class": ComposeForm
                        }, name='modal_messages_compose'),
)



import datetime
from django.test import TestCase
from django.contrib.auth.models import User
from models import Message
from utils import strip_mail
from django.utils.html import strip_tags
        
class UtilsTest(TestCase):
    def test_strip_quotes(self):
        body = """nyan nyan nyan nyan nyan
        nyan nyan nyan nyan nyan

        nyan nyan nyan nyan nyan








        2011/10/28 Nyan Cat <nyan@nyan.cat>:
         > hey guys
        > sarete il 31 dicembre con Pascal a Firenze?
        > lo spero tanto, nel caso ditemi qualcosa...
        >
        >>>
        >
        >>
        >"""
        
        body_stripped = """nyan nyan nyan nyan nyan
nyan nyan nyan nyan nyan

nyan nyan nyan nyan nyan
"""
        
        self.assertEquals(body_stripped.strip(), strip_mail(body).strip())
    
    def test_single_line_quotes(self):
        body = 'asfasf\n\nOn Thu, Dec 15, 2011 at 12:42 PM, Fabrizio S. <messaging@email.gidsy.com>wrote:\n\n>       [image: Gidsy]  New message\n>   Hi Fabrizio, Andrew M. sent you a message\n>\n> *blabla*\n> gasg\n>\n> View and reply<http://email.gidsy.com/wf/click?c=e36x8iH5CyW6UFPc7U%2FiBSpwHwOcqQc55u6Od0IAvnJWLQwR0RdOslgfJYtFkOT0&rp=7%2Bq%2FuBUXPhfnWd079jPZDJw1s3xtQcNITJcDWjO98HB8tJ6%2BYeP23y9SaOFiXvnpboQhDnEJRnrEZfRP9WnHQiL7q9Y0Plign2S9mx7i8%2Bk%3D&u=icfx5E9JS66UPX7QM9UvGw%2Fh0>\n>\n> Sincerely,\n> the *Gidsy team*<http://email.gidsy.com/wf/click?c=nun%2FbaehJTxhIK1KvYwhU5Tg16XMq0b2DKd6IxvO%2F%2Bw%3D&rp=7%2Bq%2FuBUXPhfnWd079jPZDJw1s3xtQcNITJcDWjO98HB8tJ6%2BYeP23y9SaOFiXvnpboQhDnEJRnrEZfRP9WnHQiL7q9Y0Plign2S9mx7i8%2Bk%3D&u=icfx5E9JS66UPX7QM9UvGw%2Fh1>\n>\n> This email was intended for fabrizio@gidsy.com. If you do not want to\n> receive emails like this from staging.gidsy.com<http://email.gidsy.com/wf/click?c=e36x8iH5CyW6UFPc7U%2FiBY72qxV4NIiQfC%2BfF%2BpSEec%3D&rp=7%2Bq%2FuBUXPhfnWd079jPZDJw1s3xtQcNITJcDWjO98HB8tJ6%2BYeP23y9SaOFiXvnpboQhDnEJRnrEZfRP9WnHQiL7q9Y0Plign2S9mx7i8%2Bk%3D&u=icfx5E9JS66UPX7QM9UvGw%2Fh2>anymore, then please change your Email\n> notification settings <http://notice-email-setting/>.\n>\n> Copyright \ufffd 2011 Gidsy.com, All rights reserved.\n>\n'


        body_stripped = "asfasf"
        self.assertEquals(body_stripped.strip(), strip_mail(body).strip())

    def test_strip_signature(self):
        body = 'signature test\n\nOn Fri, Dec 16, 2011 at 11:06 AM, Fabrizio Sestito <fabrizio@gidsy.com>wrote:\n\n> test\n>\n> asd\n>\n>\n> On Fri, Dec 16, 2011 at 11:05 AM, Fabrizio Sestito <fabrizio@gidsy.com>wrote:\n>\n>> hey\n>>\n>>\n>> On Thu, Dec 15, 2011 at 4:08 PM, Fabrizio S. <messaging@email.gidsy.com>wrote:\n>>\n>>>       [image: Gidsy]  New message\n>>>   Hi Fabrizio, Andrew M. sent you a message\n>>>\n>>> *sdfsdf*\n>>> sadasdasdasd\n>>>\n>>> View and reply<http://email.gidsy.com/wf/click?c=e36x8iH5CyW6UFPc7U%2FiBSpwHwOcqQc55u6Od0IAvnLUol8UpZle1eFZgQF40o%2FA&rp=7%2Bq%2FuBUXPhfnWd079jPZDJw1s3xtQcNITJcDWjO98HC6dm1r92yU0SpAJEPbb%2B6TYHFx5ZDq5B8IwoyftFTyY2YZCtQ%2F66rRPRshi2lf8V8%3D&u=wYte5RSXQ3KUXaXN31g4LQ%2Fh0>\n>>>\n>>> Sincerely,\n>>> the *Gidsy team*<http://email.gidsy.com/wf/click?c=nun%2FbaehJTxhIK1KvYwhU5Tg16XMq0b2DKd6IxvO%2F%2Bw%3D&rp=7%2Bq%2FuBUXPhfnWd079jPZDJw1s3xtQcNITJcDWjO98HC6dm1r92yU0SpAJEPbb%2B6TYHFx5ZDq5B8IwoyftFTyY2YZCtQ%2F66rRPRshi2lf8V8%3D&u=wYte5RSXQ3KUXaXN31g4LQ%2Fh1>\n>>>\n>>> This email was intended for fabrizio@gidsy.com. If you do not want to\n>>> receive emails like this from staging.gidsy.com<http://email.gidsy.com/wf/click?c=e36x8iH5CyW6UFPc7U%2FiBY72qxV4NIiQfC%2BfF%2BpSEec%3D&rp=7%2Bq%2FuBUXPhfnWd079jPZDJw1s3xtQcNITJcDWjO98HC6dm1r92yU0SpAJEPbb%2B6TYHFx5ZDq5B8IwoyftFTyY2YZCtQ%2F66rRPRshi2lf8V8%3D&u=wYte5RSXQ3KUXaXN31g4LQ%2Fh2>anymore, then please change your Email\n>>> notification settings <http://notice-email-setting/>.\n>>>\n>>> Copyright \ufffd 2011 Gidsy.com, All rights reserved.\n>>>\n>>\n>>\n>\n\n\n-- \nFabrizio Sestito\n'

        body_stripped = "signature test"
        self.assertEquals(body_stripped.strip(), strip_mail(body).strip())
        
        
    def test_no_signature(self):
        pass
        #TODO: add support for stripped html
        #body = '<div>I am some nasty html</div>'
        #body_stripped = "I am some nasty html"
        #self.assertEquals(body_stripped.strip(), strip_mail(body).strip())



from django.utils.html import strip_tags
import settings as sendgrid_settings
import logging
logger = logging.getLogger('threaded_messages')

if sendgrid_settings.THREADED_MESSAGES_USE_SENDGRID:
    from sendgrid_parse_api.signals import email_received
else:
    email_received = None

def signal_received_email(sender, sma, app_id, html, text, from_field, **kwargs):
    from utils import reply_to_thread, strip_mail # circular dependency fix
    logger.debug("Sendgrid signal receive: %s, %s, %s, %s, %s, %s"%(sender, sma, app_id,
                                                                    html, repr(text), from_field) )
    if app_id == sendgrid_settings.THREADED_MESSAGES_ID:
        body =''

        if text:
            body = text

        if not body:
            body = html

        if body:
            body = strip_tags(body)
            body = strip_mail(body)
            thread = sma.content_object
            reply_to_thread(thread, sma.user, body)

def start_listening():
    if email_received:
        logger.debug("Sendgrid start listening")
        email_received.connect(signal_received_email, dispatch_uid="thm_reply")



import datetime
import settings as sendgrid_settings
from django import forms
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext_noop
from django.contrib.auth.models import User
from models import *
from fields import CommaSeparatedUserField
from utils import reply_to_thread

if sendgrid_settings.THREADED_MESSAGES_USE_SENDGRID:
    from sendgrid_parse_api.utils import create_reply_email

notification = None
if "notification" in settings.INSTALLED_APPS:
    from notification import models as notification
    
class ComposeForm(forms.Form):
    """
    A simple default form for private messages.
    """
    recipient = CommaSeparatedUserField(label=_(u"Recipient"))
    subject = forms.CharField(label=_(u"Subject"))
    body = forms.CharField(label=_(u"Body"),
        widget=forms.Textarea(attrs={'rows': '12', 'cols':'55'}))
    
    def __init__(self, *args, **kwargs):
        recipient_filter = kwargs.pop('recipient_filter', None)
        super(ComposeForm, self).__init__(*args, **kwargs)
        if recipient_filter is not None:
            self.fields['recipient']._recipient_filter = recipient_filter
    
    def save(self, sender, send=True):
        recipients = self.cleaned_data['recipient']
        subject = self.cleaned_data['subject']
        body = self.cleaned_data['body']
        
        new_message = Message.objects.create(body=body, sender=sender)
        
        thread = Thread.objects.create(subject=subject,
                                       latest_msg=new_message,
                                       creator=sender)
        thread.all_msgs.add(new_message)

        for recipient in recipients:
            Participant.objects.create(thread=thread, user=recipient)
        
        (sender_part, created) = Participant.objects.get_or_create(thread=thread, user=sender)
        sender_part.replied_at = sender_part.read_at = datetime.datetime.now()
        sender_part.save()
        
        thread.save() #save this last, since this updates the search index
        
        #send notifications
        if send and notification:
            if sendgrid_settings.THREADED_MESSAGES_USE_SENDGRID:
                for r in recipients:
                    reply_email = create_reply_email(sendgrid_settings.THREADED_MESSAGES_ID, r, thread)
                    notification.send(recipients, "received_email", 
                                        {"thread": thread,
                                         "message": new_message}, sender=sender,
                                        from_email= reply_email.get_from_email(),
                                        headers = {'Reply-To': reply_email.get_reply_to_email()})
            else:
                notification.send(recipients, "received_email", 
                                        {"thread": thread,
                                         "message": new_message}, sender=sender)
        
        return (thread, new_message)


class ReplyForm(forms.Form):
    """
    A simple default form for private messages.
    """
    body = forms.CharField(label=_(u"Reply"),
        widget=forms.Textarea(attrs={'rows': '4', 'cols':'55'}))
    
    def save(self, sender, thread):
        body = self.cleaned_data['body']
        return reply_to_thread(thread, sender, body)



from haystack.indexes import *
from haystack import site
from models import Thread

class ThreadIndex(RealTimeSearchIndex):
    text = CharField(document=True, use_template=True)
    participants = MultiValueField()
    last_message = DateTimeField(model_attr='latest_msg__sent_at')

    def index_queryset(self):
        return Thread.objects.all()

    def prepare_participants(self, object):
        return [p.user.pk for p in object.participants.all()]

site.register(Thread, ThreadIndex)

#from haystack import indexes
#from models import Thread
#
#class ThreadIndex(indexes.SearchIndex, indexes.Indexable):
#    text = indexes.CharField(document=True, use_template=True)
#    participants = indexes.MultiValueField()
#    last_message = indexes.DateTimeField(model_attr='latest_msg__sent_at')
#
#    def index_queryset(self):
#        return Thread.objects.all()
#
#    def prepare_participants(self, object):
#        return [p.user.pk for p in object.participants.all()]
#
#    def get_model(self):
#        return Thread
#




from django.conf import settings
from django.utils.translation import ugettext_noop as _
from django.db.models import signals

if "notification" in settings.INSTALLED_APPS:
    from notification import models as notification
    def create_notice_types(app, created_models, verbosity, **kwargs):
        notification.create_notice_type("received_email", _("Received a Message"), _("Someone has sent you a message"))
    signals.post_syncdb.connect(create_notice_types, sender=notification)
else:
    print "Skipping creation of NoticeTypes (Threaded Messages) as notification app not found"


# -*- coding:utf-8 -*-
import datetime
from django.contrib.auth import load_backend, login, BACKEND_SESSION_KEY
from django.http import Http404, HttpResponseRedirect, HttpResponse
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.contrib.auth.models import User
from django.contrib.auth.decorators import login_required
from django.utils.translation import ugettext as _
from django.utils.translation import ugettext_noop
from django.core.urlresolvers import reverse
from django.db.models import Q
from django.conf import settings
from django.template.loader import render_to_string
from avatar.templatetags.avatar_tags import avatar_url
from threaded_messages.models import *
from threaded_messages.forms import ComposeForm, ReplyForm
import simplejson
from threaded_messages.models import Thread
import logging


@login_required
def inbox(request, template_name='django_messages/inbox.html'):
    """
    Displays a list of received messages for the current user.
    Optional Arguments:
        ``template_name``: name of the template to use.
    """
    only_read = request.GET.get("only_read", False)
    only_unread = request.GET.get("only_unread", False)
    only_unreplied = request.GET.get("only_unreplied", None)

    read = None
    if only_read:
        read = True
    elif only_unread:
        read = False

    if only_unreplied:
        only_unreplied = True

    thread_list = Participant.objects.inbox_for(request.user, read=read, only_unreplied=only_unreplied)
        
    return render_to_response(template_name, {
        'thread_list': thread_list,
        'only_read': only_read,
        'only_unread': only_unread,
        'only_unreplied': only_unreplied,
    }, context_instance=RequestContext(request))


@login_required
def search(request, template_name="django_messages/search.html"):
    from haystack.query import SearchQuerySet #include here, so the dependency is only needed when used
    search_term = request.GET.get("q")
    results = SearchQuerySet().filter(content=search_term,
                                    participants=request.user.pk)\
                                    .order_by('-last_message').models(Thread)
                    # leads to error in haystack: .order_by("-last_message")
    return render_to_response(template_name, {
                                  "thread_results": results,
                                  "search_term": search_term,
                                }, context_instance=RequestContext(request))
    
    
@login_required
def outbox(request, template_name='django_messages/inbox.html'):
    """
    Displays a list of sent messages by the current user.
    Optional arguments:
        ``template_name``: name of the template to use.
    """
    thread_list = Participant.objects.outbox_for(request.user)
    return render_to_response(template_name, {
        'thread_list': thread_list,
    }, context_instance=RequestContext(request))

@login_required
def trash(request, template_name='django_messages/trash.html'):
    """
    Displays a list of deleted messages. 
    Optional arguments:
        ``template_name``: name of the template to use
    Hint: A Cron-Job could periodicly clean up old messages, which are deleted
    by sender and recipient.
    """
    message_list = Participant.objects.trash_for(request.user)
    return render_to_response(template_name, {
        'message_list': message_list,
    }, context_instance=RequestContext(request))

@login_required
def compose(request, recipient=None, form_class=ComposeForm,
        template_name='django_messages/compose.html', success_url=None, recipient_filter=None):
    """
    Displays and handles the ``form_class`` form to compose new messages.
    Required Arguments: None
    Optional Arguments:
        ``recipient``: username of a `django.contrib.auth` User, who should
                       receive the message, optionally multiple usernames
                       could be separated by a '+'
        ``form_class``: the form-class to use
        ``template_name``: the template to use
        ``success_url``: where to redirect after successfull submission
    """
    recipients = []
    if request.method == "POST":
        sender = request.user
        form = form_class(data=request.POST, recipient_filter=recipient_filter)
        if form.is_valid():
            form.save(sender=request.user)
            request.user.message_set.create(
                message=_(u"Message successfully sent."))
            if success_url is None:
                success_url = reverse('messages_inbox')
            if request.GET.has_key('next'):
                success_url = request.GET['next']
            return HttpResponseRedirect(success_url)
    else:
        form = form_class()
        if recipient is not None:
            recipients = [u for u in User.objects.filter(username__in=[r.strip() for r in recipient.split('+')])]
            form.fields['recipient'].initial = recipients
    return render_to_response(template_name, {
        'form': form,
        'recipients': recipients,
    }, context_instance=RequestContext(request))


@login_required
def delete(request, thread_id, success_url=None):
    """
    Marks a message as deleted by sender or recipient. The message is not
    really removed from the database, because two users must delete a message
    before it's save to remove it completely. 
    A cron-job should prune the database and remove old messages which are 
    deleted by both users.
    As a side effect, this makes it easy to implement a trash with undelete.
    
    You can pass ?next=/foo/bar/ via the url to redirect the user to a different
    page (e.g. `/foo/bar/`) than ``success_url`` after deletion of the message.
    """
    user = request.user
    now = datetime.datetime.now()
    thread = get_object_or_404(Thread, id=thread_id)
    user_part = get_object_or_404(Participant, user=user, thread=thread)

    if request.GET.has_key('next'):
        success_url = request.GET['next']
    elif success_url is None:
        success_url = reverse('messages_inbox')
    
    user_part.deleted_at = now
    user_part.save()
    user.message_set.create(message=_(u"Conversation successfully deleted."))
    return HttpResponseRedirect(success_url)


@login_required
def undelete(request, thread_id, success_url=None):
    """
    Recovers a message from trash. This is achieved by removing the
    ``(sender|recipient)_deleted_at`` from the model.
    """
    user = request.user
    thread = get_object_or_404(Thread, id=thread_id)
    user_part = get_object_or_404(Participant, user=user, thread=thread)

    if request.GET.has_key('next'):
        success_url = request.GET['next']
    elif success_url is None:
        success_url = reverse('messages_inbox')

    user_part.deleted_at = now
    user_part.save()
    user.message_set.create(message=_(u"Conversation successfully recovered."))
    return HttpResponseRedirect(success_url)

@login_required
def view(request, thread_id, form_class=ReplyForm,
        success_url=None, recipient_filter=None, template_name='django_messages/view.html'):
    """
    Shows a single message.``message_id`` argument is required.
    The user is only allowed to see the message, if he is either 
    the sender or the recipient. If the user is not allowed a 404
    is raised. 
    If the user is the recipient and the message is unread 
    ``read_at`` is set to the current datetime.
    """    

    user = request.user
    thread = get_object_or_404(Thread, id=thread_id)
  
    """
    Reply stuff
    """
    if request.method == "POST":
        form = form_class(request.POST)
        if form.is_valid():
            form.save(sender=user, thread=thread)
            request.user.message_set.create(
                message=_(u"Reply successfully sent."))
            if success_url is None:
                success_url = reverse('messages_detail', args=(thread.id,))
            return HttpResponseRedirect(success_url)
    else:
        form = form_class()

    now = datetime.datetime.now()
    participant = get_object_or_404(Participant, thread=thread, user=request.user)
    message_list = []
    # in this view we want the last message last
    for message in thread.all_msgs.all().order_by("sent_at"):
        unread = True
        if participant.read_at and message.sent_at <= participant.read_at:
            unread = False
        message_list.append((message,unread,))
    participant.read_at = now
    participant.save()
    return render_to_response(template_name, {
        'thread': thread,
        'message_list': message_list,
        'form': form,
        'participant': participant,
    }, context_instance=RequestContext(request))


@login_required
def batch_update(request, success_url=None):
    """
    Gets an array of message ids which can be either deleted or marked as
    read/unread
    """
    if request.method == "POST":
        ids = request.POST.getlist("batchupdateids")
        if ids:
            threads = Thread.objects.filter(pk__in=ids)
            for thread in threads:
                participant = thread.participants.filter(user=request.user)
                if participant:
                    participant = participant[0]
                    if request.POST.get("action") == "read":
                        participant.read_at = datetime.datetime.now()
                    elif request.POST.get("action") == "delete":
                        participant.deleted_at = datetime.datetime.now()
                    elif request.POST.get("action") == "unread":
                        participant.read_at = None
                    participant.save()
        else:
            raise Http404
        
    else:
        # this should only happen when hacked or developer uses wrong, therefore
        # return simple message
        return HttpResponse("Only Post allowed", code=400)
        
    if success_url:
        return HttpResponseRedirect(success_url)
    else:
        # either go to last page, or to inbox as fallback
        referer = request.META.get('HTTP_REFERER', None)
        if referer:
            return HttpResponseRedirect(referer)
        else:
            return HttpResponseRedirect(reverse("messages_inbox"))
            
            

@login_required
def message_ajax_reply(request, thread_id,
                  template_name="django_messages/message_list_view.html"):
    thread = get_object_or_404(Thread, id=thread_id)
    if request.POST:
        form = ReplyForm(request.POST)
        if form.is_valid():
            try:
                (thread, new_message) = form.save(sender=request.user, thread=thread)
            except Exception, e:
                logging.exception(e)
                return HttpResponse(status=500, content="Message could not be sent")
                
            return render_to_response(template_name,{
                "message": new_message,
            }, context_instance=RequestContext(request))
        else:
            return HttpResponse(status=400, content="Invalid Form")


@login_required
def recipient_search(request):
    term = request.GET.get("term")
    users = User.objects.filter(Q(first_name__icontains=term)|
                                Q(last_name__icontains=term)|
                                Q(username__icontains=term)|
                                Q(email__icontains=term))
    if request.GET.get("format") == "json":
        data = []
        for user in users:
            avatar_img_url = avatar_url(user, size=50)
            data.append({"id": user.username,
                         "url": reverse("profile_detail",args=(user.username,)),
                         "name": "%s %s"%(user.first_name, user.last_name),
                         "img": avatar_img_url})

        return HttpResponse(simplejson.dumps(data),
                            mimetype='application/json')



# -*- coding:utf-8 -*-
import re
import settings as tm_settings
from models import Message, Participant
from django.conf import settings
from django.contrib.sites.models import Site
from django.utils.encoding import force_unicode
from django.utils.text import wrap
from django.utils.translation import ugettext_lazy as _
from django.template import Context, loader
from django.template.loader import render_to_string, get_template
from django.template import Context
import HTMLParser
from lxml.html.clean import Cleaner
import datetime

if "notification" in settings.INSTALLED_APPS:
    from notification import models as notification
    
# favour django-mailer but fall back to django.core.mail
if tm_settings.THREADED_MESSAGES_USE_SENDGRID:
    import sendgrid_parse_api

if "mailer" in settings.INSTALLED_APPS:
    from mailer import send_mail
else:
    from django.core.mail import send_mail


def open_message_thread(recipients, subject, template,
                        sender, context={}, send=True, message=None):
    body = ''
    if template:
        t = get_template(template)
        body = t.render(Context({}))
    else:
        body = message
        
    from forms import ComposeForm #temporary here to remove circular dependence
    compose_form = ComposeForm(data={
        "recipient": recipients,
        "subject": subject,
        "body": body
    })
    if compose_form.is_valid():
        (thread, new_message) = compose_form.save(sender=sender, send=send)
        
    return (thread, new_message)


def reply_to_thread(thread,sender, body):  
    new_message = Message.objects.create(body=body, sender=sender)
    new_message.parent_msg = thread.latest_msg
    thread.latest_msg = new_message
    thread.all_msgs.add(new_message)
    thread.replied = True
    thread.save()
    new_message.save()
    
    recipients = []
    for participant in thread.participants.all():
        participant.deleted_at = None
        participant.save()
        if sender != participant.user: # dont send emails to the sender!
            recipients.append(participant.user)
    
    sender_part = Participant.objects.get(thread=thread, user=sender)
    sender_part.replied_at = sender_part.read_at = datetime.datetime.now()
    sender_part.save()
    
    if notification:
        for r in recipients:
            if tm_settings.THREADED_MESSAGES_USE_SENDGRID:
                reply_email = sendgrid_parse_api.utils.create_reply_email(tm_settings.THREADED_MESSAGES_ID, r, thread)
                notification.send(recipients, "received_email", 
                                        {"thread": thread,
                                         "message": new_message}, sender=sender,
                                        from_email=reply_email.get_from_email(), 
                                        headers = {'Reply-To': reply_email.get_reply_to_email()})
            else:
                notification.send([r], "received_email", 
                                    {"thread": thread,
                                     "message": new_message}, sender=sender)
        
    return (thread, new_message)


def get_lines(body):
    body = body.replace('\r', ' ')
    lines = [x.strip() for x in body.splitlines(True)]
    return lines


def strip_mail(body):

    custom_line_no = None
    
    lines = get_lines(body)
    
    has_signature = False
    for l in lines:
        if l.strip().startswith('>'):
            has_signature = True
            
    # strip signature -- only if there is a signature. otherwise all is stripped
    if has_signature:
        for l in reversed(lines):
            lines.remove(l)
            if l.strip().startswith('>'):
                break

    # strip quotes
    for i,l in enumerate(lines):
        if l.lstrip().startswith('>'):
            if not custom_line_no:
                custom_line_no = i-1
                popme = custom_line_no
                while not lines[popme]:
                    lines.pop(popme)
                    popme -=1
                if re.search(r'^.*?([0-1][0-9]|[2][0-3]):([0-5][0-9]).*?$', lines[popme]) or re.search(r'[\w-]+@([\w-]+\.)+[\w-]+', lines[popme]):
                    lines.pop(popme)
                break

    stripped_lines = [s for s in lines if not s.lstrip().startswith('>')]

    # strip last empty string in the list if it exists
    if not stripped_lines[-1]: stripped_lines.pop()

    # stripped message
    return ('\n').join(stripped_lines)



from django import template

register = template.Library()

@register.filter
# truncate after a certain number of characters
def truncchar(value, arg):
    if len(value) < arg:
        return value
    else:
        return value[:arg] + '...'



from django import forms
from django.conf import settings
from django.utils.translation import gettext_lazy as _
from django.contrib import admin
from django.contrib.auth.models import User, Group
from threaded_messages.models import *


class MessageAdmin(admin.ModelAdmin):
    list_display = ('sender', 'sent_at','body')
    
admin.site.register(Message, MessageAdmin)
admin.site.register(Thread)
admin.site.register(Participant)



from django.template import Library, Node, TemplateSyntaxError
from threaded_messages.models import inbox_count_for

class InboxOutput(Node):
    def __init__(self, varname=None):
        self.varname = varname
        
    def render(self, context):
        try:
            user = context['user']
            count = inbox_count_for(user)
        except (KeyError, AttributeError):
            count = ''
        if self.varname is not None:
            context[self.varname] = count
            return ""
        else:
            return "%s" % (count)
        
def do_print_inbox_count(parser, token):
    """
    A templatetag to show the unread-count for a logged in user.
    Returns the number of unread messages in the user's inbox.
    Usage::
    
        {% load inbox %}
        {% inbox_count %}
    
        {# or assign the value to a variable: #}
        
        {% inbox_count as my_var %}
        {{ my_var }}
        
    """
    bits = token.contents.split()
    if len(bits) > 1:
        if len(bits) != 3:
            raise TemplateSyntaxError, "inbox_count tag takes either no arguments or exactly two arguments"
        if bits[1] != 'as':
            raise TemplateSyntaxError, "first argument to inbox_count tag must be 'as'"
        return InboxOutput(bits[2])
    else:
        return InboxOutput()

register = Library()     
register.tag('inbox_count', do_print_inbox_count)



import datetime
from django.db import models
from django.conf import settings
from django.db.models import signals
from django.db.models.query import QuerySet
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _
from django.db.models import F, Q
from django.db.models import Avg, Max, Min, Count

from listeners import start_listening
start_listening()

class MessageManager(models.Manager):
    
    def inbox_for(self, user, read=None, only_unreplied=None):
        """
        Returns all messages that were received by the given user and are not
        marked as deleted.
        """
        inbox = self.filter(
            user=user,
            deleted_at__isnull=True,
        )
        if read != None:
            if read == True:
                # read messages have read_at set to a later value then last message of the thread
                inbox = inbox.exclude(read_at__isnull=True)\
                            .filter(read_at__gt=F("thread__latest_msg__sent_at"))
            else:
                # unread threads are the ones that either have not been read at all or before the last message arrived
                inbox = inbox.filter(Q(read_at__isnull=True)
                                    |Q(read_at__lt=F("thread__latest_msg__sent_at")))

        if only_unreplied != None:
            if only_unreplied == True:
                inbox = inbox.filter(Q(replied_at__isnull=True)
                                    |Q(replied_at__lt=F("thread__latest_msg__sent_at")))
                
        return inbox
    
    def outbox_for(self, user):
        """
        Returns all messages that were sent by the given user and are not
        marked as deleted.
        """
        return self.filter(
            user=user,
            replied_at__isnull=False,
            deleted_at__isnull=True,
        )
    
    def trash_for(self, user):
        """
        Returns all messages that were either received or sent by the given
        user and are marked as deleted.
        """
        return self.filter(
            user=user,
            deleted_at__isnull=False,
        )


class Message(models.Model):
    """
    A private message from user to user
    """
    body = models.TextField(_("body"))
    sender = models.ForeignKey(User, related_name='sent_messages', blank=True, null=True, verbose_name=_("sender"))
    parent_msg = models.ForeignKey('self', related_name='next_messages', blank=True, null=True, verbose_name=_("parent message"))
    sent_at = models.DateTimeField(_("sent at"), auto_now_add=True)
    
    def __unicode__(self):
        return "%s - %s" % (str(self.sender), self.sent_at)
    
    def save(self, **kwargs):
        if not self.id:
            self.sent_at = datetime.datetime.now()
        super(Message, self).save(**kwargs)
    
    class Meta:
        ordering = ['-sent_at']
        verbose_name = _("Message")
        verbose_name_plural = _("Messages")


class Thread(models.Model):
    """
    A linear conversation between two or more Users
    """
    subject = models.CharField(_("Subject"), max_length=120)
    latest_msg = models.ForeignKey(Message, related_name='thread_latest', verbose_name=_("Latest message"))
    all_msgs = models.ManyToManyField(Message, related_name='thread', verbose_name=_("Messages"))
    # the following fields are used to filter out messages that have not been replied to in the inbox
    creator = models.ForeignKey(User, related_name='created_threads', verbose_name=_("creator"))
    replied = models.BooleanField(editable=False, default=False)
    
    def __unicode__(self):
        return self.subject
    
    def get_absolute_url(self):
        return ('messages_detail', [self.id])
    get_absolute_url = models.permalink(get_absolute_url)
    
    class Meta:
        ordering = ['latest_msg']
        verbose_name = _("Thread")
        verbose_name_plural = _("Threads")
    
    
class Participant(models.Model):
    """
    Thread manager for each participant
    """
    thread = models.ForeignKey(Thread, related_name='participants', verbose_name=_("message thread"))
    user = models.ForeignKey(User, related_name='threads', verbose_name=_("participant users"))
    read_at = models.DateTimeField(_("read at"), null=True, blank=True)
    replied_at = models.DateTimeField(_("replied at"), null=True, blank=True)
    deleted_at = models.DateTimeField(_("deleted at"), null=True, blank=True)
    
    objects = MessageManager()
    
    def new(self):
        """returns whether the recipient has read the message or not"""
        if self.read_at is None or self.read_at < self.thread.latest_msg.sent_at:
            return True
        return False

    def replied(self):
        """returns whether the recipient has replied the message or not"""
        if self.replied_at is None \
            or self.replied_at < self.thread.latest_msg.sent_at:
            return True
        return False
    
    def last_other_sender(self):
        """returns the last sender thats not the viewing user. if nobody
            besides you sent a message to the thread we take a random one
        """
        message = self.thread.all_msgs.exclude(sender=self.user)
        if message:
            return message[0].sender
        else:
            others = self.others()
            if others:
                return others[0].user
        return None
    
    def others(self):
        """returns the other participants of the thread"""
        return self.thread.participants.exclude(user=self.user)

    def get_next(self):
        try:
            participation = Participant.objects.inbox_for(self.user).filter(thread__latest_msg__sent_at__gt=\
                                                                           self.thread.latest_msg.sent_at).reverse()[0]
            return participation
        except:
            return None


    def get_previous(self):
        try:
            participation = Participant.objects.inbox_for(self.user).filter(thread__latest_msg__sent_at__lt=\
                                                                            self.thread.latest_msg.sent_at)[0]
            return participation
        except:
            return None
        
    def __unicode__(self):
        return "%s - %s" % (str(self.user), self.thread.subject)
    
    class Meta:
        ordering = ['thread']
        verbose_name = _("participant")
        verbose_name_plural = _("participants")


def inbox_count_for(user):
    """
    returns the number of unread messages for the given user but does not
    mark them seen
    """
    return Participant.objects.inbox_for(user, read=False).count()




from distutils.core import setup

setup(
    name='django-threaded-messages',
    version=__import__('threaded_messages').__version__,
    description='User-to-user threaded messaging system (similar to facebook) for Django',
    author='Philipp Wassibauer, John Debs',
    author_email='phil@gidsy.com',
    url='https://github.com/philippWassibauer/django-threaded-messages',
    download_url='https://github.com/philippWassibauer/django-threaded-messages',
    packages=(
        'threaded_messages',
        'threaded_messages.templatetags',
    ),
    classifiers=(
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Utilities',
        'Framework :: Django',
    ),
)


"""
Based on http://www.djangosnippets.org/snippets/595/
by sopelkin
"""

from django import forms
from django.forms import widgets
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _


class CommaSeparatedUserInput(widgets.Input):
    input_type = 'text'
    
    def render(self, name, value, attrs=None):
        if value is None:
            value = ''
        elif isinstance(value, (list, tuple)):
            value = (', '.join([user.username for user in value]))
        return super(CommaSeparatedUserInput, self).render(name, value, attrs)


class CommaSeparatedUserField(forms.Field):
    widget = CommaSeparatedUserInput
    
    def __init__(self, *args, **kwargs):
        recipient_filter = kwargs.pop('recipient_filter', None)
        self._recipient_filter = recipient_filter
        super(CommaSeparatedUserField, self).__init__(*args, **kwargs)
        
    def clean(self, value):
        super(CommaSeparatedUserField, self).clean(value)
        if not value:
            return ''
        if isinstance(value, (list, tuple)):
            return value
        
        #test if last char is a seperator, some tokenizers are not that smart
        if value[-1] == ',':
            value = value[0:-1]
            
        names = set(value.split(','))
        names_set = set([name.strip() for name in names])
        users = list(User.objects.filter(username__in=names_set))
        unknown_names = names_set ^ set([user.username for user in users])
        
        recipient_filter = self._recipient_filter
        invalid_users = []
        if recipient_filter is not None:
            for r in users:
                if recipient_filter(r) is False:
                    users.remove(r)
                    invalid_users.append(r.username)
        
        if unknown_names or invalid_users:
            raise forms.ValidationError(_(u"The following usernames are incorrect: %(users)s") % {'users': ', '.join(list(unknown_names)+invalid_users)})
        
        return users
